/*
 *  Intelligent Platform Management Interface driver for Linux 2.x
 *
 *  (c) Copyright 1999  San Mehat & VA Linux Systems
 *        1382 Bordeaux Dr.
 *        Sunnyvale, California
 *        94089
 *
 *  http://www.valinux.com
 *
 *  This driver is provided under the GNU public license, incorporated
 *  herein by reference. The driver is provided without warranty or
 *  support.
 *
 *
 */

#include <linux/config.h>

#define IO 0xca2
#define ISA_BMC_STATUS    (IO + 1)
#define ISA_BMC_COMMAND   (IO + 1)
#define ISA_BMC_DATA_IN       (IO + 0)
#define ISA_BMC_DATA_OUT      (IO + 0)

/* State bits based on S1 & S0 below */
#define ISA_STATE_MASK        0xC0
#define ISA_IDLE_STATE        0x00
#define ISA_READ_STATE        0x40
#define ISA_WRITE_STATE       0x80
#define ISA_ERROR_STATE       0xC0

/* Status Register Bits */
#define ISA_S1_FLAG           0x80
#define ISA_S0_FLAG           0x40
#define ISA_CD_FLAG           0x08
#define ISA_SMS_MSG_FLAG      0x04
#define ISA_IBF_FLAG          0x02
#define ISA_OBF_FLAG          0x01

/* SMS Transfer Stream Control Codes */
#define ISA_GET_STATUS_ABORT   0x60
#define ISA_WRITE_START        0x61
#define ISA_WRITE_END          0X62
#define ISA_READ               0X68

#define MAX_ISA_LENGTH  35

#define ISA_TIMEOUT 1000

typedef struct ipmi_ksc_set_watchdog
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;

  unsigned char timer_use         :3;
  unsigned char res1              :4;
  unsigned char dontlog           :1;

  unsigned char timeout_action    :3;
  unsigned char res2              :1;
  unsigned char pre_irq           :3;
  unsigned char res3              :1;

  unsigned char   pretimeout_interval;

  unsigned char tuefc_res1        :1;
  unsigned char tuefc_biosfrb2    :1;
  unsigned char tuefc_biospost    :1;
  unsigned char tuefc_osload      :1;
  unsigned char tuefc_smsos       :1;
  unsigned char tuefc_oem         :1;
  unsigned char tuefc_res2        :1;
  unsigned char tuefc_res3        :1;

  unsigned short initial_count;
  } IPMI_KCS_SET_WATCHDOG;

typedef struct get_watchdog_response
  {
  unsigned char cc           __attribute__ ((packed));

  unsigned char timer_use    :3;
  unsigned char res1         :3;
  unsigned char timer_status :1;
  unsigned char sel_log      :1;

  unsigned char timeout_act  :3;
  unsigned char res2         :1;
  unsigned char pre_irq_act  :3;
  unsigned char res3         :1;

  unsigned char pre_timeout __attribute__ ((packed));

  unsigned char timer_use_xp  __attribute__ ((packed));

  unsigned short init_count  __attribute__ ((packed));
  unsigned short current_count __attribute__ ((packed));
  } GET_WATCHDOG_RESPONSE;

typedef struct ipmi_ksc_reset_watchdog
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;
  } IPMI_KCS_RESET_WATCHDOG;

typedef struct sensor_cmd
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;
  unsigned char sensor_number;
  } SENSOR_CMD;

/*
 * Structure for a generic cmd that requires no args
 */
typedef struct ipmi_kcs_generic_cmd
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;
  } KCS_GENERIC_CMD;

/*
 * FRU area info response
 */
typedef struct kcs_fru_area_info
  {
  unsigned char    cc    __attribute__ ((packed));
  unsigned short  size  __attribute__ ((packed));

#define ACCESSMODE_BYTE  0
#define ACCESSMODE_WORD  1
  unsigned char    accessmode:1;
  unsigned char    reserved:7;
  } FRU_AREA_INFO;

typedef struct kcs_fru_data_req
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;

  unsigned short  offset  __attribute__ ((packed));
  unsigned char    count    __attribute__ ((packed));
  } FRU_DATA_REQ;

typedef struct fru_common_header
  {     
  unsigned char common_header_format_version;
  unsigned char internal_use_offset;
  unsigned char chassis_info_offset;
  unsigned char board_area_offset;
  unsigned char product_info_offset;
  unsigned char multirecord_area_offset;
  unsigned char pad;
  unsigned char checksum;
  } FRU_COMMON_HEADER;

typedef struct type_length
  {
  unsigned char  num_bytes  :6;
  unsigned char  type_code  :2;
  } TL;

typedef struct ipmi_kcs_add_sel_cmd
  {
  unsigned char lun               :2;
  unsigned char netfn             :6;

  unsigned char cmd;

  unsigned short  record_id    __attribute__ ((packed));
  unsigned char    record_type  __attribute__ ((packed));
  unsigned int    timestamp    __attribute__ ((packed));
  unsigned short  mfid        __attribute__ ((packed));

  unsigned char    data[6]      __attribute__ ((packed));
  } ADD_SEL_CMD;

/* GET_DEVICE_ID RESPONSE */
typedef struct device_id_response
  {
  unsigned char cc;
  unsigned char device_id;
  
  unsigned char device_revision         :4;
  unsigned char reserved                :3;
  unsigned char provides_sdr            :1;
  
  unsigned char major_firmware_revision :7;
  #define NORMAL_OPERATION  0
  #define DEVICE_BUSY       1
  unsigned char device_available        :1;
  
  unsigned char minor_firmware_revision;
  
  unsigned char ipmi_version_major      :4;
  unsigned char ipmi_version_minor      :4;
  
  unsigned char supports_sensor_device  :1;
  unsigned char supports_sdr_device     :1;
  unsigned char supports_sel_device     :1;
  unsigned char supports_fru_device     :1;
  unsigned char supports_ipmb_receiver  :1;
  unsigned char supports_ipmb_generator :1;
  unsigned char supports_bridge         :1;
  unsigned char supports_chassis_device :1;
  
  unsigned char manufacturer_id1;
  unsigned char manufacturer_id2;
  unsigned char manufacturer_id3;
  
  unsigned short product_id;
  } DEVICE_ID_RESPONSE;

/*
 * The Netfn codes
 */
#define CHASSIS_REQUEST   0x00
#define CHASSIS_RESPONSE  0x01
#define BRIDGE_REQUEST    0x02
#define BRIDGE_RESPONSE   0x03
#define SENSOR_REQUEST    0x04
#define SENSOR_RESPONSE   0x05
#define APP_REQUEST       0x06
#define APP_RESPONSE      0x07
#define FIRMWARE_REQUEST  0x08
#define FIRMWARE_RESPONSE 0x09
#define STORAGE_REQUEST   0x0A
#define STORAGE_RESPONSE  0x0B

/*
 * The cmds
 */
#define CMD_GET_FRU_INV_AREA_INFO   0x10
#define CMD_READ_FRU_INV_DATA       0x11
#define CMD_WRITE_FRU_INV_DATA      0x12
#define CMD_ADD_SEL_ENTRY           0x44
#define CMD_WATCHDOG_SET            0x24
#define CMD_WATCHDOG_GET            0x25
#define CMD_WATCHDOG_RESET          0x22
#define CMD_GET_DEVICE_ID           0x01

#define VALINUX_MFID  0x1B58
